@@ -50,5 +50,7 @@ namespace tnt
     const std::string pragma = "Pragma:";
     const std::string expires = "Expires:";
     const std::string userAgent = "User-Agent:";
+    const std::string wwwAuthenticate = "WWW-Authenticate:";
+    const std::string authorization = "Authorization:";
   }
 }
