@@ -202,6 +202,12 @@ namespace tnt
     return HTTP_MOVED_TEMPORARILY;
   }
 
+  unsigned HttpReply::notAuthorized(const std::string& realm)
+  {
+    setHeader(httpheader::wwwAuthenticate, "Basic realm=" + realm + '"');
+    return HTTP_UNAUTHORIZED;
+  }
+
   void HttpReply::setContentLengthHeader(size_t size)
   {
     std::ostringstream s;
@@ -245,7 +251,9 @@ namespace tnt
   void HttpReply::setCookie(const std::string& name, const Cookie& value)
   {
     log_debug("setCookie(\"" << name << "\",\"" << value.getValue() << "\")");
-    httpcookies.setCookie(name, value);
+    tnt::Cookie cookie(value);
+    cookie.setPath("/");
+    httpcookies.setCookie(name, cookie);
   }
 
   bool HttpReply::keepAlive() const
