@@ -23,6 +23,7 @@
 #include <sstream>
 #include <cxxtools/log.h>
 #include <cxxtools/thread.h>
+#include <cxxtools/base64stream.h>
 #include <sys/socket.h>
 #include <arpa/inet.h>
 #include <tnt/sessionscope.h>
@@ -264,11 +265,30 @@ namespace tnt
     if (!locale_init)
     {
       static const std::string LANG = "LANG";
-      static const std::locale stdlocale("");
+      static std::locale stdlocale;
+      static bool stdlocale_init = false;
+
       typedef std::map<std::string, std::locale> locale_map_type;
       static locale_map_type locale_map;
       static cxxtools::Mutex locale_monitor;
 
+      if (!stdlocale_init)
+      {
+        cxxtools::MutexLock lock(locale_monitor);
+        if (!stdlocale_init)
+        {
+          stdlocale_init = true;
+          try
+          {
+            stdlocale = std::locale("");
+          }
+          catch (const std::exception& e)
+          {
+            log_warn("error initializing standard-locale - using locale \"C\"");
+          }
+        }
+      }
+
       locale_init = true;
 
       log_debug("HttpRequest::getLocale() " << qparam.dump());
@@ -333,6 +353,29 @@ namespace tnt
     return encoding;
   }
 
+  std::string HttpRequest::getUser() const
+  {
+    if (username.empty() && hasHeader(httpheader::authorization))
+    {
+      std::istringstream authHeader(getHeader(httpheader::authorization));
+      while (authHeader && authHeader.get() != ' ')
+        ;
+      cxxtools::Base64istream in(authHeader);
+      std::getline(in, username, ':');
+      std::getline(in, password);
+
+      log_debug("username \"" << username << "\" password \"" << password << '"');
+    }
+    return username;
+  }
+
+  bool HttpRequest::verifyPassword(const std::string& password_) const
+  {
+    getUser();
+    log_debug("verify password \"" << password_ << "\" for username \"" << username << "\" password \"" << password << '"');
+    return password == password_;
+  }
+
   namespace
   {
     class CompareNoCase
