@@ -2,21 +2,22 @@
  * Copyright (C) 2003-2005 Tommi Maekitalo
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
- * is provided AS IS, WITHOUT ANY WARRANTY; without even the implied
- * warranty of MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and
- * NON-INFRINGEMENT.  See the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
+
 #include "tnt/ecppc/generator.h"
 #include <tnt/stringescaper.h>
 #include <iterator>
@@ -35,16 +36,16 @@ namespace tnt
     ////////////////////////////////////////////////////////////////////////
     // Generator
     //
-    Generator::Generator(const std::string& classname, const std::string& ns)
-      : singleton(true),
-        raw(false),
-        maincomp(classname, ns),
+    Generator::Generator(const std::string& classname)
+      : raw(false),
+        maincomp(classname),
         haveCloseComp(false),
         closeComp(classname),
         currentComp(&maincomp),
         externData(false),
         compress(false),
-        c_time(0)
+        c_time(0),
+        linenumbersEnabled(true)
     {
       time_t cur;
       time(&cur);
@@ -133,15 +134,6 @@ namespace tnt
       pre += m.str();
     }
 
-    void Generator::onDeclare(const std::string& code)
-    {
-      singleton = false;
-      std::ostringstream m;
-      printLine(m);
-      m << code << '\n';
-      declare += m.str();
-    }
-
     void Generator::onInit(const std::string& code)
     {
       std::ostringstream m;
@@ -184,11 +176,6 @@ namespace tnt
       currentComp->addEndCall(curline, curfile, comp);
     }
 
-    void Generator::onDeclareShared(const std::string& code)
-    {
-      declare_shared += code;
-    }
-
     void Generator::onShared(const std::string& code)
     {
       shared += code;
@@ -277,8 +264,6 @@ namespace tnt
              "#include <tnt/convert.h>\n";
       if (!componentclass.empty())
         out << "#include \"" << componentclass << ".h\"\n";
-      if (!baseclass.empty())
-        out << "#include \"" << baseclass << ".h\"\n";
     }
 
     void Generator::getPre(std::ostream& out) const
@@ -292,28 +277,13 @@ namespace tnt
     {
       out << "namespace\n"
              "{\n";
-
-      if (!maincomp.getNs().empty())
-        out << "namespace " << maincomp.getNs() << "\n"
-               "{\n";
-
-      out << '\n';
     }
 
     void Generator::getNamespaceEnd(std::ostream& out) const
     {
-      if (!maincomp.getNs().empty())
-        out << "} // namespace " << maincomp.getNs() << '\n';
       out << "} // namespace\n\n";
     }
 
-    void Generator::getDeclareShared(std::ostream& out) const
-    {
-      out << "// <%declare_shared>\n"
-          << declare_shared 
-          << "// </%declare_shared>\n";
-    }
-
     void Generator::getClassDeclaration(std::ostream& out) const
     {
       out << "class " << maincomp.getName() << " : public ";
@@ -321,14 +291,9 @@ namespace tnt
         out << "tnt::EcppComponent";
       else
         out << componentclass;
-      if (!baseclass.empty())
-        out << ", public " << baseclass;
       out << "\n"
              "{\n"
              "    " << maincomp.getName() << "& main()  { return *this; }\n\n" 
-             "    // <%declare>\n"
-          << declare
-          << "    // </%declare>\n\n"
              "  protected:\n"
              "    ~" << maincomp.getName() << "();\n\n"
              "  public:\n"
@@ -337,8 +302,6 @@ namespace tnt
       if (haveCloseComp)
         out << "    unsigned endTag(tnt::HttpRequest& request, tnt::HttpReply& reply,\n"
                "                    cxxtools::QueryParams& qparam);\n";
-      out << "    void drop();\n\n";
-
       if (!attr.empty())
         out << "    std::string getAttribute(const std::string& name,\n"
                "      const std::string& def = std::string()) const;\n\n";
@@ -370,8 +333,7 @@ namespace tnt
              "#include <tnt/httpheader.h>\n"
              "#include <tnt/http.h>\n"
              "#include <tnt/data.h>\n"
-             "#include <tnt/componentfactory.h>\n"
-             "#include <tnt/componentguard.h>\n";
+             "#include <tnt/componentfactory.h>\n";
       if (hasScopevars())
         out << "#include <tnt/objecttemplate.h>\n"
                "#include <tnt/objectptr.h>\n";
@@ -391,66 +353,43 @@ namespace tnt
 
     void Generator::getFactoryDeclaration(std::ostream& code) const
     {
-      const char* factoryBase = singleton ? "tnt::SingletonComponentFactory" : "tnt::ComponentFactory";
-      code << "class " << maincomp.getName() << "Factory : public " << factoryBase << "\n"
-              "{\n"
-              "  public:\n"
-              "    " << maincomp.getName() << "Factory(const std::string& componentName)\n"
-              "      : " << factoryBase << "(componentName)\n"
-              "      { }\n"
-              "    virtual tnt::Component* doCreate(const tnt::Compident& ci,\n"
-              "      const tnt::Urlmapper& um, tnt::Comploader& cl);\n";
-      if (!configs.empty())
-        code << "    virtual void doConfigure(const tnt::Tntconfig& config);\n";
-      code << "};\n"
-              "\n"
-              "tnt::Component* " << maincomp.getName() << "Factory::doCreate(const tnt::Compident& ci,\n"
-              "  const tnt::Urlmapper& um, tnt::Comploader& cl)\n"
-              "{\n"
-              "  return new " << maincomp.getName() << "(ci, um, cl);\n"
-              "}\n\n";
-      if (!configs.empty())
+      if (configs.empty())
       {
-        code << "void " << maincomp.getName() << "Factory::doConfigure(const tnt::Tntconfig& config)\n"
+        code << "static tnt::ComponentFactoryImpl<" << maincomp.getName() << "> "
+             << maincomp.getName() << "Factory(\"" << maincomp.getName() << "\");\n\n";
+      }
+      else
+      {
+        code << "class " << maincomp.getName() << "Factory : public tnt::ComponentFactoryImpl<" << maincomp.getName() << ">\n"
+                "{\n"
+                "  public:\n"
+                "    " << maincomp.getName() << "Factory()\n"
+                "      : tnt::ComponentFactoryImpl<" << maincomp.getName() << ">(\"" << maincomp.getName() << "\")\n"
+                "      { }\n"
+                "    tnt::Component* doCreate(const tnt::Compident& ci,\n"
+                "      const tnt::Urlmapper& um, tnt::Comploader& cl);\n"
+                "    virtual void doConfigure(const tnt::Tntconfig& config);\n"
+                "};\n\n"
+                "tnt::Component* " << maincomp.getName() << "Factory::doCreate(const tnt::Compident& ci,\n"
+                "  const tnt::Urlmapper& um, tnt::Comploader& cl)\n"
+                "{\n"
+                "  return new " << maincomp.getName() << "(ci, um, cl);\n"
+                "}\n\n"
+                "void " << maincomp.getName() << "Factory::doConfigure(const tnt::Tntconfig& config)\n"
                 "{\n"
                 "  // <%config>\n";
         for (variable_declarations::const_iterator it = configs.begin();
              it != configs.end(); ++it)
           it->getConfigInit(code, maincomp.getName());
         code << "  // </%config>\n"
-                "}\n\n";
+                "}\n\n"
+                "static " << maincomp.getName() << "Factory factory;\n\n";
       }
-
-      getNamespaceEnd(code);
-
-      std::string ns = maincomp.getNs().empty() ? std::string() : (maincomp.getNs() + "::");
-      std::string componentName = maincomp.getName();
-      std::string factoryType = ns + maincomp.getName() + "Factory";
-      std::string factoryName = (maincomp.getNs().empty() ? std::string() : (maincomp.getNs() + "__"))
-                              + componentName + factorySuffix;
-
-      code << "extern \"C\" {\n"
-              "  " << factoryType << ' ' << factoryName << "(\"" << componentName << "\");\n"
-              "}\n\n";
-
-      getNamespaceStart(code);
-
-      code << factoryType << "& factory = " << factoryName << ";\n\n";
     }
 
     void Generator::getCppBody(std::ostream& code) const
     {
       std::string classname = maincomp.getName();
-      std::string ns_classname;
-      std::string ns_classname_dot;
-
-      if (!maincomp.getNs().empty())
-      {
-        ns_classname = maincomp.getNs() + "::";
-        ns_classname_dot = maincomp.getNs() + '.';
-      }
-      ns_classname += classname;
-      ns_classname_dot += classname;
 
       getFactoryDeclaration(code);
 
@@ -529,7 +468,7 @@ namespace tnt
       code << "// <%shared>\n"
            << shared
            << "// </%shared>\n\n";
-      if (!singleton && !configs.empty())
+      if (!configs.empty())
         code << "bool config_init = false;\n";
       code << "// <%config>\n";
       for (variable_declarations::const_iterator it = configs.begin();
@@ -584,7 +523,7 @@ namespace tnt
       if (!data.empty())
       {
         if (externData)
-          code << "  tnt::DataChunks data(getData(request, rawData));\n";
+          code << "  tnt::DataChunks data(getData(request, qparam, rawData));\n";
         else
           code << "  tnt::DataChunks data(rawData);\n";
       }
@@ -632,10 +571,6 @@ namespace tnt
 
       if (haveCloseComp)
         closeComp.getDefinition(code, externData);
-      code << "void " << classname << "::drop()\n"
-              "{\n"
-              "  factory.drop(this);\n"
-              "}\n\n";
 
       if (!attr.empty())
       {
@@ -657,29 +592,23 @@ namespace tnt
 
     void Generator::printLine(std::ostream& out) const
     {
-      out << "#line " << (curline + 1) << " \"" << curfile << "\"\n";
+      if (linenumbersEnabled)
+        out << "#line " << (curline + 1) << " \"" << curfile << "\"\n";
     }
 
     void Generator::getHeader(std::ostream& header, const std::string& filename) const
     {
-      std::string ns_classname_;
-      const std::string classname = maincomp.getName();
-      const std::string ns = maincomp.getNs();
-      if (ns.empty())
-        ns_classname_ = classname;
-      else
-        ns_classname_ = ns + '_' + classname;
+      std::string classname = maincomp.getName();
 
       getIntro(header, filename);
-      header << "#ifndef ECPP_COMPONENT_" << ns_classname_ << "_H\n"
-                "#define ECPP_COMPONENT_" << ns_classname_ << "_H\n\n";
+      header << "#ifndef ECPP_COMPONENT_" << classname << "_H\n"
+                "#define ECPP_COMPONENT_" << classname << "_H\n\n";
 
       getHeaderIncludes(header);
       header << '\n';
       getPre(header);
       header << '\n';
       getNamespaceStart(header);
-      getDeclareShared(header);
       getClassDeclaration(header);
       getNamespaceEnd(header);
 
@@ -689,50 +618,36 @@ namespace tnt
 
     void Generator::getCpp(std::ostream& code, const std::string& filename) const
     {
-      std::string ns_classname_slash;
-      std::string ns_classname_dot;
-
-      if (!maincomp.getNs().empty())
-      {
-        ns_classname_slash = maincomp.getNs() + '/';
-        ns_classname_dot = maincomp.getNs() + '.';
-      }
-      ns_classname_slash += maincomp.getName();
-      ns_classname_dot += maincomp.getName();
-
       getIntro(code, filename);
       getCppIncludes(code);
 
-      code << "#include \"" << ns_classname_slash << ".h\"\n\n"
-              "log_define(\"component." << ns_classname_dot << "\")\n\n"
-              "template <typename T> inline void use(const T&) { }\n\n";
+      code << "#include \"" << maincomp.getName() << ".h\"\n\n"
+              "log_define(\"component." << maincomp.getName() << "\")\n\n";
 
       getNamespaceStart(code);
+
+      code << "template <typename T> inline void use(const T&) { }\n\n";
+
       getCppBody(code);
       getNamespaceEnd(code);
     }
 
     void Generator::getCppWoHeader(std::ostream& code, const std::string& filename) const
     {
-      std::string ns_classname_dot;
-      if (!maincomp.getNs().empty())
-        ns_classname_dot = maincomp.getNs() + '.';
-      ns_classname_dot += maincomp.getName();
-
       getIntro(code, filename);
 
       getHeaderIncludes(code);
       getCppIncludes(code);
 
-      code << "log_define(\"component." << ns_classname_dot << "\")\n\n"
-              "template <typename T> inline void use(const T&) { }\n\n";
+      code << "log_define(\"component." << maincomp.getName() << "\")\n\n";
 
       getPre(code);
       code << '\n';
 
       getNamespaceStart(code);
 
-      getDeclareShared(code);
+      code << "template <typename T> inline void use(const T&) { }\n\n";
+
       getClassDeclaration(code);
 
       getCppBody(code);
